@@ -1113,12 +1113,105 @@ prefs_set_status_icon_theme_cb(GtkComboBox *combo_box, gpointer user_data)
 	}
 }
 
+#ifdef USE_HILDON
+static void
+theme_chosen_cb(const char *theme_type, char *filename)
+{
+/*
+	struct theme_info {
+		gchar *type;
+		gchar *extension;
+		gchar *original_name;
+	};
+*/
+	struct theme_info *ti = g_new0(struct theme_info, 1);
+	char *tmp = NULL;
+
+	g_print("theme_chosen_cb: %s\n", filename);
+
+	if (ti) {
+		ti->type = g_strdup(theme_type);
+		ti->extension = g_strdup(g_strrstr(filename, "."));
+
+		tmp = g_strrstr(filename, G_DIR_SEPARATOR_S);
+		ti->original_name = g_strdup(tmp ? tmp : filename);
+
+		theme_install_theme(filename, ti);
+	}
+}
+
+static void
+add_theme_button_clicked(GtkWidget *btn, const char *theme_type)
+{
+	purple_request_file(prefs, _("Install Theme"), NULL, FALSE,
+	                    G_CALLBACK(theme_chosen_cb), NULL,
+	                    NULL, NULL, NULL, (gpointer)theme_type);
+}
+
+static GtkWidget *
+make_add_theme_button(const char *theme_type)
+{
+	GtkWidget *btn = g_object_new(HILDON_TYPE_BUTTON,
+		"visible",     TRUE,
+		"title",       _("Add"),
+		"size",        HILDON_SIZE_FINGER_HEIGHT,
+		"arrangement", HILDON_BUTTON_ARRANGEMENT_VERTICAL,
+		NULL);
+
+	g_signal_connect(G_OBJECT(btn), "clicked", (GCallback)add_theme_button_clicked, (gpointer)theme_type);
+
+	return btn;
+}
+#endif /* USE_HILDON */
+
+static GtkWidget *
+add_theme_prefs_combo(GtkWidget *vbox, GtkSizeGroup *combo_sg, GtkSizeGroup *label_sg, GtkListStore *theme_store,
+                      GCallback combo_box_cb, gpointer combo_box_cb_user_data, 
+                      const char *label_str, const char *prefs_path, const char *theme_type)
+{
+	GtkWidget *label;
+	GtkWidget *combo_box = NULL;
+	char *full_prefs_path = g_strdup_printf(PIDGIN_PREFS_ROOT "%s", prefs_path);
+#ifdef USE_HILDON
+	GtkWidget *themesel_hbox = gtk_vbox_new(FALSE, PIDGIN_HIG_BOX_SPACE);
+	GtkWidget *hbox = gtk_hbox_new(FALSE, PIDGIN_HIG_BOX_SPACE);
+#else /* !USE_HILDON */
+	GtkWidget *themesel_hbox = gtk_hbox_new(FALSE, PIDGIN_HIG_BOX_SPACE);
+#endif /* USE_HILDON */
+
+	label = gtk_label_new(label_str);
+	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
+	gtk_size_group_add_widget(label_sg, label);
+	gtk_box_pack_start(GTK_BOX(themesel_hbox), label, FALSE, FALSE, 0);
+
+	combo_box = prefs_build_theme_combo_box(theme_store,
+						purple_prefs_get_string(full_prefs_path),
+						theme_type);
+	g_signal_connect(G_OBJECT(combo_box), "changed",
+						(GCallback)combo_box_cb, combo_box_cb_user_data);
+	gtk_size_group_add_widget(combo_sg, combo_box);
+#ifdef USE_HILDON
+	gtk_widget_set_size_request(combo_box, 600, -1);
+	gtk_box_pack_start(GTK_BOX(hbox), combo_box, TRUE, TRUE, 0);
+	gtk_box_pack_start(GTK_BOX(hbox), make_add_theme_button(theme_type), FALSE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(themesel_hbox), hbox, TRUE, TRUE, 0);
+#else /* !USE_HILDON */
+	gtk_box_pack_start(GTK_BOX(themesel_hbox), combo_box, TRUE, TRUE, 0);
+#endif /* USE_HILDON */
+
+	gtk_box_pack_start(GTK_BOX(vbox), themesel_hbox, FALSE, FALSE, 0);
+	g_free(full_prefs_path);
+
+	return combo_box;
+}
+
 static GtkWidget *
 theme_page(void)
 {
-	GtkWidget *ret, *vbox;
+#ifndef USE_HILDON
 	GtkWidget *label;
-	GtkWidget *themesel_hbox;
+#endif /* USE_HILDON */
+	GtkWidget *ret, *vbox;
 	GtkSizeGroup *label_sg = gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL);
 	GtkSizeGroup *combo_sg = gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL);
 
@@ -1126,7 +1219,9 @@ theme_page(void)
 	gtk_container_set_border_width (GTK_CONTAINER (ret), PIDGIN_HIG_BORDER);
 
 	vbox = pidgin_make_frame(ret, _("Theme Selections"));
-
+#ifdef USE_HILDON
+	g_object_set(G_OBJECT(vbox), "spacing", HILDON_MARGIN_TRIPLE, NULL);
+#else /* !USE_HILDON */
 	/* Instructions */
 	label = gtk_label_new(_("Select a theme that you would like to use from "
 							"the lists below.\nNew themes can be installed by "
@@ -1137,78 +1232,30 @@ theme_page(void)
 
 	gtk_box_pack_start(GTK_BOX(vbox), label, TRUE, FALSE, 0);
 	gtk_widget_show(label);
-
+#endif /* USE_HILDON */
 	/* Buddy List Themes */
-	themesel_hbox = gtk_hbox_new(FALSE, PIDGIN_HIG_BOX_SPACE);
-
-	label = gtk_label_new(_("Buddy List Theme:"));
-	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
-	gtk_size_group_add_widget(label_sg, label);
-	gtk_box_pack_start(GTK_BOX(themesel_hbox), label, FALSE, FALSE, 0);
-
-	prefs_blist_themes_combo_box = prefs_build_theme_combo_box(prefs_blist_themes,
-						purple_prefs_get_string(PIDGIN_PREFS_ROOT "/blist/theme"),
-						"blist");
-	g_signal_connect(G_OBJECT(prefs_blist_themes_combo_box), "changed",
-						(GCallback)prefs_set_blist_theme_cb, NULL);
-	gtk_size_group_add_widget(combo_sg, prefs_blist_themes_combo_box);
-	gtk_box_pack_start(GTK_BOX(themesel_hbox), prefs_blist_themes_combo_box, TRUE, TRUE, 0);
-
-	gtk_box_pack_start(GTK_BOX(vbox), themesel_hbox, FALSE, FALSE, 0);
+	prefs_blist_themes_combo_box = add_theme_prefs_combo(
+		vbox, combo_sg, label_sg, prefs_blist_themes,
+		(GCallback)prefs_set_blist_theme_cb, NULL, 
+		_("Buddy List Theme:"), "/blist/theme", "blist");
 
 	/* Status Icon Themes */
-	themesel_hbox = gtk_hbox_new(FALSE, PIDGIN_HIG_BOX_SPACE);
-
-	label = gtk_label_new(_("Status Icon Theme:"));
-	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
-	gtk_size_group_add_widget(label_sg, label);
-	gtk_box_pack_start(GTK_BOX(themesel_hbox), label, FALSE, FALSE, 0);
-
-	prefs_status_themes_combo_box = prefs_build_theme_combo_box(prefs_status_icon_themes,
-						purple_prefs_get_string(PIDGIN_PREFS_ROOT "/status/icon-theme"),
-						"icon");
-	g_signal_connect(G_OBJECT(prefs_status_themes_combo_box), "changed",
-						(GCallback)prefs_set_status_icon_theme_cb, NULL);
-	gtk_size_group_add_widget(combo_sg, prefs_status_themes_combo_box);
-	gtk_box_pack_start(GTK_BOX(themesel_hbox), prefs_status_themes_combo_box, TRUE, TRUE, 0);
-
-	gtk_box_pack_start(GTK_BOX(vbox), themesel_hbox, FALSE, FALSE, 0);
+	prefs_status_themes_combo_box = add_theme_prefs_combo(
+		vbox, combo_sg, label_sg, prefs_status_icon_themes,
+		(GCallback)prefs_set_status_icon_theme_cb, NULL,
+		_("Status Icon Theme:"), "/status/icon-theme", "icon");
 
 	/* Sound Themes */
-	themesel_hbox = gtk_hbox_new(FALSE, PIDGIN_HIG_BOX_SPACE);
-
-	label = gtk_label_new(_("Sound Theme:"));
-	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
-	gtk_size_group_add_widget(label_sg, label);
-	gtk_box_pack_start(GTK_BOX(themesel_hbox), label, FALSE, FALSE, 0);
-
-	prefs_sound_themes_combo_box = prefs_build_theme_combo_box(prefs_sound_themes,
-						purple_prefs_get_string(PIDGIN_PREFS_ROOT "/sound/theme"),
-						"sound");
-	g_signal_connect(G_OBJECT(prefs_sound_themes_combo_box), "changed",
-						(GCallback)prefs_set_sound_theme_cb, NULL);
-	gtk_size_group_add_widget(combo_sg, prefs_sound_themes_combo_box);
-	gtk_box_pack_start(GTK_BOX(themesel_hbox), prefs_sound_themes_combo_box, TRUE, TRUE, 0);
-
-	gtk_box_pack_start(GTK_BOX(vbox), themesel_hbox, FALSE, FALSE, 0);
+	prefs_sound_themes_combo_box = add_theme_prefs_combo(
+		vbox, combo_sg, label_sg, prefs_sound_themes,
+		(GCallback)prefs_set_sound_theme_cb, NULL,
+		_("Sound Theme:"), "/sound/theme", "sound");
 
 	/* Smiley Themes */
-	themesel_hbox = gtk_hbox_new(FALSE, PIDGIN_HIG_BOX_SPACE);
-
-	label = gtk_label_new(_("Smiley Theme:"));
-	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
-	gtk_size_group_add_widget(label_sg, label);
-	gtk_box_pack_start(GTK_BOX(themesel_hbox), label, FALSE, FALSE, 0);
-
-	prefs_smiley_themes_combo_box = prefs_build_theme_combo_box(prefs_smiley_themes,
-						purple_prefs_get_string(PIDGIN_PREFS_ROOT "/smileys/theme"),
-						"smiley");
-	g_signal_connect(G_OBJECT(prefs_smiley_themes_combo_box), "changed",
-						(GCallback)prefs_set_smiley_theme_cb, NULL);
-	gtk_size_group_add_widget(combo_sg, prefs_smiley_themes_combo_box);
-	gtk_box_pack_start(GTK_BOX(themesel_hbox), prefs_smiley_themes_combo_box, TRUE, TRUE, 0);
-
-	gtk_box_pack_start(GTK_BOX(vbox), themesel_hbox, FALSE, FALSE, 0);
+	prefs_smiley_themes_combo_box = add_theme_prefs_combo(
+		vbox, combo_sg, label_sg, prefs_smiley_themes,
+		(GCallback)prefs_set_smiley_theme_cb, NULL,
+		_("Smiley Theme:"), "/smileys/theme", "smiley");
 
 	/* Custom sort so "none" theme is at top of list */
 	gtk_tree_sortable_set_sort_func(GTK_TREE_SORTABLE(prefs_smiley_themes),
@@ -2304,6 +2351,7 @@ sound_changed3_cb(const char *name, PurplePrefType type,
 
 	gtk_widget_set_sensitive(hbox,
 			!strcmp(method, "automatic") ||
+			!strcmp(method, "alsa") ||
 			!strcmp(method, "esd"));
 }
 #endif /* USE_GSTREAMER */
